/*
** ----------------------------------------------------------------
** idp.c  -  Modulo que calcula Integral de Divisao de Polinomios.
**
** ----------------------------------------------------------------
*/

/*@{*/
/**
** @name Modulo que calcula integral de divisao de polinomios
**
** @date    25-Out-2005
** @author  Alexandre Lopes e Luiz F. Martha
** @version 1.0
**
** Este modulo contem uma funcao que calcula a integral 
** unidimensional de divisao de polinomios em uma faixa
** arbitraria.
**
** Revisoes:
**
*/

/*
** ---------------------------------------------------------------
** Definicoes e variaveis globais:
*/
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "idp.h"

/*
** ---------------------------------------------------------------
** Definicoes e variaveis locais:
*/

/*
** ---------------------------------------------------------------
** Funcoes locais:
*/

/*==========================  my_pow  ==========================*/

/** Funcao que calcula a potencia inteira de um numero real.
 **
 **  @param x           - numero real para obter potencia
 **  @param n           - potencia inteira
 **
 ** @return
 **  resultado da potencia
 **/
static double my_pow( double x, int n )
{
 int i;
 double p = 1.0;
 for( i = 1; i <= n; i++ )
  p *= x;
 return( p );
}

/*====================  integraDivPoliLocal  ===================*/

/** Funcao que realiza a integracao da divisao de dois polinomios
 ** no intervalo [a,b].
 ** A funcao utiliza integracao numerica de Gauss com quatro
 ** pontos de integracao.
 **
 **  @param coefPoliNumerador   - coeficientes do polinomio do numerador
 **                               ( primeiro posio  o grau )
 **  @param coefPoliDenominador - coeficientes do polinomio do denominador
 **                               ( primeiro posio  o grau )
 **  @param a           - limite inferior de integracao
 **  @param b           - limite superior de integracao
 **
 ** @return
 **  resultado da integracao
 **/
static double integraDivPoliLocal( double *coefPoliNumerador, 
                                   double *coefPoliDenominador, 
                                   double a, double b )
{
 /** abscissas de Gauss considerando xi variando de 0 a 1 */ 
 static double gaussAbscissas[] = { 0.069431844202974,
                                    0.330009478207572,
                                    0.669990521792428,
                                    0.930568155797026 };

 /** pesos de Gauss considerando xi variando de 0 a 1 */
 static double gaussPesos[] = { 0.173927422568727,
                                0.326072577431273,
                                0.326072577431273,
                                0.173927422568727 };

 /** numero de pontos de Gauss */
 static int numPontosGauss = 4;

 /** grau do numerador */
 int grauNumerador = (int)coefPoliNumerador[0];

 /** grau do denominador */
 int grauDenominador = (int)coefPoliDenominador[0];

 /** variavel auxiliar para armazenar o valor do numerador */
 double poliNumerador;

 /** variavel auxiliar para armazenar o valor do denominador */
 double poliDenominador;

 /** variavel auxiliar para armazenar o valor da integral */
 double integral = 0.0;

 int i;
 int j;

  for( i = 0; i < numPontosGauss; i++ )
  {
   poliNumerador = 0.0;
   poliDenominador = 0.0;
   for( j = 0; j <= grauNumerador; j++ )
   {
    poliNumerador += coefPoliNumerador[j+1] * 
                     my_pow( a*(1.0-gaussAbscissas[i]) + b*gaussAbscissas[i], 
                             grauNumerador-j );
   }
   for( j = 0; j <= grauDenominador; j++ )
   {
    poliDenominador += coefPoliDenominador[j+1] * 
                       my_pow( a*(1.0-gaussAbscissas[i]) + b*gaussAbscissas[i],
                               grauDenominador-j );
   }
   integral += (b-a) * (poliNumerador/poliDenominador) * gaussPesos[i];
  }
 return( integral );
}


/*
** ---------------------------------------------------------------
** Funcoes exportadas:
*/

/*=====================  idpIntegraDivPoli  ====================*/

double idpIntegraDivPoli( double *coefPoliNumerador, 
                          double *coefPoliDenominador, 
                          double a, double b )
{
 double integral = 0.0;
 double integralParcial1 = 0.0;
 double integralParcial2 = 0.0;
 static double erro = 1E-8;

 integral = integraDivPoliLocal( coefPoliNumerador, 
                                 coefPoliDenominador, a , b );
 integralParcial1 = integraDivPoliLocal( coefPoliNumerador,
                                         coefPoliDenominador, a, (a+b)*0.5 );
 integralParcial2 = integraDivPoliLocal( coefPoliNumerador,
                                         coefPoliDenominador,(a+b)*0.5, b );
 if( fabs( integral - (integralParcial1 + integralParcial2) ) < erro )
  return( integral );
 else
  return( 
     idpIntegraDivPoli( coefPoliNumerador,coefPoliDenominador,a, (a+b)*0.5 ) +
     idpIntegraDivPoli( coefPoliNumerador,coefPoliDenominador, (a+b)*0.5, b ) );
}

/*@}*/
