/*
** ----------------------------------------------------------------
** mis.c  -  Module for the misula functions.
**
** ----------------------------------------------------------------
*/

/*@{*/
/**
** @name Module for the misula functions
**
** @date    23-Dez-2005
** @author  Paula Vilela
** @version 1.0
**
** This file contains the implementation of the functions
** that compute fix-end reaction moments and stiffness
** coefficients for a beam element with a cross secton 
** that has a fourth order polynomial variation of moment 
** of inertia.
**
** See definitions of public functions in hearder file "mis.h".
**
** Revisions:
** - Report modifications here (including name and date).
**
*/

/*
** ---------------------------------------------------------------
** Global variables and symbols:
*/
#include <stdlib.h>
#include <stdio.h>
#include "mis.h"
#include "idp.h"

/*
** ---------------------------------------------------------------
** Local variables and symbols:
*/

/*
** ---------------------------------------------------------------
** Private functions:
*/

/*==========================mis_J_x0=============================*/

/** This function computes the values of coefficient of x^0 in the 
 ** equation of moment of inertia, for a beam element with a cross 
 ** section that has a fourth order polynomial variation.
 **
 **  @param J0   - initial moment of inertia value                (in)
 **
 ** @return 
 ** Value of coefficient of x^0 in the denominator of the equations  
 **/
static double mis_J_x0( double J0 )
{
 return( J0 );
}

/*==========================mis_J_x1=============================*/

/** This function computes the values of coefficient of x^1 in the 
 ** equation of moment of inertia, for a beam element with a cross 
 ** section that has a fourth order polynomial variation.
 **
 **  @param len - beam element length (span)                      (in)
 **  @param J0  - initial moment of inertia value                 (in)
 **  @param J1  - moment of inertia at 1/4 of beam span           (in)
 **  @param J2  - moment of inertia at 1/2 of beam span           (in)
 **  @param J3  - moment of inertia at 3/4 of beam span           (in)
 **  @param J4  - final moment of inertia value                   (in)
 **
 ** @return 
 ** Value of coefficient of x^1 in the denominator of the equations  
 **/
static double mis_J_x1( double len, double J0, double J1,
                        double J2, double J3, double J4 )
{
 return( -(25*J0-48*J1+3*J4+36*J2-16*J3)/(len*3) );
}

/*==========================mis_J_x2=============================*/

/** This function computes the values of coefficient of x^2 in the 
 ** equation of moment of inertia, for a beam element with a cross 
 ** section that has a fourth order polynomial variation.
 **
 **  @param len - beam element length (span)                      (in)
 **  @param J0  - initial moment of inertia value                 (in)
 **  @param J1  - moment of inertia at 1/4 of beam span           (in)
 **  @param J2  - moment of inertia at 1/2 of beam span           (in)
 **  @param J3  - moment of inertia at 3/4 of beam span           (in)
 **  @param J4  - final moment of inertia value                   (in)
 **
 ** @return 
 ** Value of coefficient of x^2 in the denominator of the equations  
 **/
static double mis_J_x2( double len, double J0, double J1,
                        double J2, double J3, double J4 )
{
 return( 2*(35*J0+11*J4+114*J2-56*J3-104*J1)/(len*len*3) );
}

/*==========================mis_J_x3=============================*/

/** This function computes the values of coefficient of x^3 in the 
 ** equation of moment of inertia, for a beam element with a cross 
 ** section that has a fourth order polynomial variation.
 **
 **  @param len - beam element length (span)                      (in)
 **  @param J0  - initial moment of inertia value                 (in)
 **  @param J1  - moment of inertia at 1/4 of beam span           (in)
 **  @param J2  - moment of inertia at 1/2 of beam span           (in)
 **  @param J3  - moment of inertia at 3/4 of beam span           (in)
 **  @param J4  - final moment of inertia value                   (in)
 **
 ** @return 
 ** Value of coefficient of x^3 in the denominator of the equations  
 **/
static double mis_J_x3( double len, double J0, double J1,
                        double J2, double J3, double J4 )
{
 return( -16*(5*J0+3*J4+24*J2-14*J3-18*J1)/(len*len*len*3) );
}

/*==========================mis_J_x4=============================*/

/** This function computes the values of coefficient of x^4 in the 
 ** equation of moment of inertia, for a beam element with a cross 
 ** section that has a fourth order polynomial variation.
 **
 **  @param len - beam element length (span)                      (in)
 **  @param J0  - initial moment of inertia value                 (in)
 **  @param J1  - moment of inertia at 1/4 of beam span           (in)
 **  @param J2  - moment of inertia at 1/2 of beam span           (in)
 **  @param J3  - moment of inertia at 3/4 of beam span           (in)
 **  @param J4  - final moment of inertia value                   (in)
 **
 ** @return 
 ** Value of coefficient of x^4 in the denominator of the equations  
 **/
static double mis_J_x4( double len, double J0, double J1,
                        double J2, double J3, double J4 )
{
 return( 32*(J0-4*J1+6*J2-4*J3+J4)/(len*len*len*len*3) );
}

/*=======================  misSystemMatCoef  ======================*/

/** This function computes the values of matrix coefficients of
 ** the system of equations of the conjugate beam.  These 
 ** coefficients are common to all the fundamental solutions 
 ** (rotation stiffness coefficients, fix-end moments for linearly
 ** distributed loads, etc.) that use the conjugate beam anology.
 **
 **  @param len - beam element length (span)                      (in)
 **  @param J0  - initial moment of inertia value                 (in)
 **  @param J1  - moment of inertia at 1/4 of beam span           (in)
 **  @param J2  - moment of inertia at 1/2 of beam span           (in)
 **  @param J3  - moment of inertia at 3/4 of beam span           (in)
 **  @param J4  - final moment of inertia value                   (in)
 **  @param a, b, c, d - values of the system matrix coefficients (out)
 **
 **/
static void misSystemMatCoef( double len, double J0, double J1, 
                              double J2, double J3, double J4, 
                              double *a, double *b, double *c, double *d )
{
 double num_0[2];    /* numerator of integrals of x^0 in func. a */
 double num_1[3];    /* numerator of integrals of x^1 in func. a, b, c */
 double num_2[4];    /* numerator of integrals of x^2 in func. c and d */
 double den[6];      /* denominator polynomial coefficients of all func.*/
 double i_0;
 double i_1;
 double i_2;

 num_0[0] = (double)0;
 num_0[1] = 1.0;
   
 num_1[0] = (double)1;
 num_1[1] = 1.0;
 num_1[2] = 0.0;

 num_2[0] = (double)2;
 num_2[1] = 1.0;
 num_2[2] = 0.0;
 num_2[3] = 0.0;
 
 den[0] = (double)4;
 den[1] = mis_J_x4( len, J0, J1, J2, J3, J4 );
 den[2] = mis_J_x3( len, J0, J1, J2, J3, J4 );
 den[3] = mis_J_x2( len, J0, J1, J2, J3, J4 );
 den[4] = mis_J_x1( len, J0, J1, J2, J3, J4 );
 den[5] = mis_J_x0( J0 );
 
 i_0 = idpIntegraDivPoli( num_0, den, 0.0, len);
 i_1 = idpIntegraDivPoli( num_1, den, 0.0, len );  
 i_2 = idpIntegraDivPoli( num_2, den, 0.0, len );
 
 *a = i_1/len - i_0;
 *b = i_1/len;
 *c = i_2/len - i_1;
 *d = i_2/len;
}
 
/*==========================misFixEnd_q_e_x1=============================*/

/** This function computes the values of coefficients of x^1 and x^2 of the 
 ** numerator in the integrals:
 ** e(x) = Mq/J
 ** f(x) = Mq*x/J
 ** for the computation of fix-end moments for linearly distributed load.
 **
 **  @param len     - beam element length                         (in)
 **  @param qa      - initial transversal load value              (in)
 **  @param qb      - final transversal load value                (in)
 **
 ** @return 
 ** Value of coefficient of x^1 in the numerator in the equation e(x) and
 ** the value of coefficient of x^2 in the numerator in the equation f(x)
 **/
static double misFixEnd_q_e_x1( double len, double qa, double qb )
{
 return( -len*(2*qa+qb)/6.0 );
}

/*==========================misFixEnd_q_e_x2=============================*/

/** This function computes the values of coefficients of x^2 and x^3 of the 
 ** numerator in the integrals:
 ** e(x) = Mq/J
 ** f(x) = Mq*x/J
 ** for the computation of fix-end moments for linearly distributed load.
 **
 **  @param qa      - initial transversal load value              (in)
 **
 ** @return 
 ** Value of coefficient of x^2 in the numerator in the equation e(x) and
 ** the value of coefficient of x^3 in the numerator in the equation f(x)
 **/
static double misFixEnd_q_e_x2( double qa )
{
 return( qa/2.0 );
}

/*==========================misFixEnd_q_e_x3=============================*/

/** This function computes the values of coefficients of x^3 and x^4 of the 
 ** numerator in the integrals:
 ** e(x) = Mq/J
 ** f(x) = Mq*x/J
 ** for the computation of fix-end moments for linearly distributed load.
 **
 **  @param len     - beam element length                         (in)
 **  @param qa      - initial transversal load value              (in)
 **  @param qb      - final transversal load value                (in)
 **
 ** @return 
 ** Value of coefficient of x^3 in the numerator in the equation e(x) and
 ** the value of coefficient of x^4 in the numerator in the equation f(x)
 **/
static double misFixEnd_q_e_x3( double len, double qa, double qb )
{
 return( -(qa-qb)/(len*6.0) );
}


/*
** ---------------------------------------------------------------
** Public functions:
*/

/*=======================  misRotStiffCoef  ======================*/
 
void misRotStiffCoef( double len, double J0, double J1, 
                      double J2, double J3, double J4,
                      double *Ka, double *Kb, double *tab, double *tba )
{
 double Ma;       /* initial fix-end reaction moments */
 double Mb;       /* final fix-end reaction moments */
 double a;        /* integral of a func. of conjugate beam equil. eqn.*/
 double b;        /* integral of b func. of conjugate beam equil. eqn.*/
 double c;        /* integral of c func. of conjugate beam equil. eqn.*/
 double d;        /* integral of d func. of conjugate beam equil. eqn.*/
 double e;        /* value of e load term of conjugate beam equil.eqn.*/
 double f;        /* value of f load term of conjugate beam equil.eqn.*/

 misSystemMatCoef( len, J0, J1, J2, J3, J4, &a, &b, &c, &d );

 e = 1.0;
 f = 0.0;
 
 Ma = (b*f - d*e)/(d*a - b*c);
 Mb = (c*e - f*a)/(d*a - b*c);
   
 *Ka = Ma;
 *tab = Mb/Ma;

 e = -1.0;
 f = -len;
 
 Ma = (b*f - d*e)/(d*a - b*c);
 Mb = (c*e - f*a)/(d*a - b*c);
   
 *Kb = Mb;
 *tba = Ma/Mb;
}

/*======================== misFixEnd_q =============================*/

void misFixEnd_q( double len, double qa, double qb, 
                  double J0, double J1, double J2, double J3, double J4,
                  double *Ma, double *Mb )
{
 double a;        /* integral of a func. of conjugate beam equil. eqn.*/
 double b;        /* integral of b func. of conjugate beam equil. eqn.*/
 double c;        /* integral of c func. of conjugate beam equil. eqn.*/
 double d;        /* integral of d func. of conjugate beam equil. eqn.*/
 double e;        /* value of e load term of conjugate beam equil.eqn.*/
 double f;        /* value of f load term of conjugate beam equil.eqn.*/
 double den[6];   /* denominator polynomial coefficients of all func.*/
 double n_e[5];   /* numerator polynomial coefficients of func. e */
 double n_f[6];   /* numerator polynomial coefficients of func. f */
 
 misSystemMatCoef( len, J0, J1, J2, J3, J4, &a, &b, &c, &d );
   
 den[0] = (double)4;
 den[1] = mis_J_x4( len, J0, J1, J2, J3, J4 );
 den[2] = mis_J_x3( len, J0, J1, J2, J3, J4 );
 den[3] = mis_J_x2( len, J0, J1, J2, J3, J4 );
 den[4] = mis_J_x1( len, J0, J1, J2, J3, J4 );
 den[5] = mis_J_x0( J0 );
 
 n_e[0] = (double)3;
 n_e[1] = misFixEnd_q_e_x3( len, qa, qb );
 n_e[2] = misFixEnd_q_e_x2( qa );
 n_e[3] = misFixEnd_q_e_x1( len, qa, qb );
 n_e[4] = 0.0 ;
 
 n_f[0] = (double)4;
 n_f[1] = misFixEnd_q_e_x3( len, qa, qb );
 n_f[2] = misFixEnd_q_e_x2( qa );
 n_f[3] = misFixEnd_q_e_x1( len, qa, qb );
 n_f[4] = 0.0 ;
 n_f[5] = 0.0;
 
 e = idpIntegraDivPoli( n_e, den, 0.0, len );
 f = idpIntegraDivPoli( n_f, den, 0.0, len );
 
 *Ma = (b*f - d*e)/(a*d - b*c);
 *Mb = (c*e - a*f)/(a*d - b*c);
}

/* ========================= misFixEnd_dv ==========================*/

void misFixEnd_dv( double len, double J0, 
                   double J1, double J2, double J3, double J4, 
                   double *Ma, double *Mb )
{
 double a;              /* integral of a func. of conjugate beam equil. eqn.*/
 double b;              /* integral of b func. of conjugate beam equil. eqn.*/
 double c;              /* integral of c func. of conjugate beam equil. eqn.*/
 double d;              /* integral of d func. of conjugate beam equil. eqn.*/
 double e = 0.0;        /* value of e load term of conjugate beam equil.eqn.*/
 double f = -1.0;       /* value of f load term of conjugate beam equil.eqn.*/

 misSystemMatCoef( len, J0, J1, J2, J3, J4, &a, &b, &c, &d );
 	
 *Ma = (b*f - d*e)/(a*d - b*c);
 *Mb = (c*e - a*f)/(a*d - b*c);
}

/* ========================== misFixEnd_dtheta =========================*/

void misFixEnd_dtheta( double len, double xs, double J0, 
                       double J1, double J2, double J3,
                       double J4, double *Ma, double *Mb )
{
 double a;               /* integral of a func. of conjugate beam equil. eqn.*/
 double b;               /* integral of b func. of conjugate beam equil. eqn.*/
 double c;               /* integral of c func. of conjugate beam equil. eqn.*/
 double d;               /* integral of d func. of conjugate beam equil. eqn.*/
 double e = 1.0;         /* value of e load term of conjugate beam equil.eqn.*/
 double f = xs;          /* value of f load term of conjugate beam equil.eqn.*/

 misSystemMatCoef( len, J0, J1, J2, J3, J4, &a, &b, &c, &d );
 	
 *Ma = (b*f - d*e)/(a*d - b*c);
 *Mb = (c*e - a*f)/(a*d - b*c);
}
 
/* ======================== misDispl_MaMb ================================*/

double misDispl_MaMb( double xs, double len, 
                      double J0, double J1, double J2, double J3, double J4,
                      double Ma, double Mb )
{
 double num_0[2];    /* numerator of integrals of x^0 in func. a */
 double num_1[3];    /* numerator of integrals of x^1 in func. a, b, c */
 double num_2[4];    /* numerator of integrals of x^2 in func. c and d */
 double den[6];      /* denominator polynomial coefficients of all func.*/
 double i_0;
 double i_1;
 double i_2;

 num_0[0] = (double)0;
 num_0[1] = 1.0;
   
 num_1[0] = (double)1;
 num_1[1] = 1.0;
 num_1[2] = 0.0;

 num_2[0] = (double)2;
 num_2[1] = 1.0;
 num_2[2] = 0.0;
 num_2[3] = 0.0;
 
 den[0] = (double)4;
 den[1] = mis_J_x4( len, J0, J1, J2, J3, J4 );
 den[2] = mis_J_x3( len, J0, J1, J2, J3, J4 );
 den[3] = mis_J_x2( len, J0, J1, J2, J3, J4 );
 den[4] = mis_J_x1( len, J0, J1, J2, J3, J4 );
 den[5] = mis_J_x0( J0 );
 
 i_0 = idpIntegraDivPoli( num_0, den, 0.0, xs );
 i_1 = idpIntegraDivPoli( num_1, den, 0.0, xs );  
 i_2 = idpIntegraDivPoli( num_2, den, 0.0, xs );
   
 return( Ma*(i_1*xs/len - i_2 - i_0*xs + i_1) +  Mb*(i_1*xs/len - i_2) );
}

/* ======================= misDispl_q ===================================*/

double misDispl_q( double xs, double len, 
                   double J0, double J1, double J2, double J3, double J4,
                   double qa, double qb )
{
 double num[6];
 double den[6];      /* denominator polynomial coefficients */
 double integ;
   
 num[0] = (double)4;
 num[1] = - misFixEnd_q_e_x3( len, qa, qb );
 num[2] = - misFixEnd_q_e_x2( qa ) + (misFixEnd_q_e_x3( len, qa, qb ))*xs;
 num[3] = - misFixEnd_q_e_x1( len, qa, qb ) + (misFixEnd_q_e_x2( qa ))*xs;
 num[4] = - (misFixEnd_q_e_x1( len, qa, qb ))*xs;
 num[5] =  0.0;
 
 den[0] = (double)4;
 den[1] = mis_J_x4( len, J0, J1, J2, J3, J4 );
 den[2] = mis_J_x3( len, J0, J1, J2, J3, J4 );
 den[3] = mis_J_x2( len, J0, J1, J2, J3, J4 );
 den[4] = mis_J_x1( len, J0, J1, J2, J3, J4 );
 den[5] = mis_J_x0( J0 );
 
 integ = idpIntegraDivPoli( num, den, 0.0, xs );
 
 return( integ );
}
 
/* ======================== misLinearInterpDepth ===========================*/

void misLinearInterpDepth( double h0, double h4,
                           double *h1, double *h2, double *h3 )
{
 *h1 = h0*0.75 + h4*0.25;
 *h2 = h0*0.50 + h4*0.50;
 *h3 = h0*0.25 + h4*0.75;
}
 
/* ======================== misParabInterpDepth0 ===========================*/

void misParabInterpDepth0( double h0, double h4,
                           double *h1, double *h2, double *h3 )
{
 *h1 = (h0-h4)/16.0 + (h4-h0)/2.0 + h0;
 *h2 = (h0-h4)/4.0 + h4;
 *h3 = 9.0*(h0-h4)/16.0 + 3.0*(h4-h0)/2.0 + h0;
}
 
/* ======================== misParabInterpDepth1 ===========================*/

void misParabInterpDepth1( double h0, double h4,
                           double *h1, double *h2, double *h3 )
{
 *h1 = (h4-h0)/16.0 + h0;
 *h2 = (h4-h0)/4.0 + h0;
 *h3 = 9.0*(h4-h0)/16.0 + h0;
}

/*@}*/

