/*
** ---------------------------------------------------------------
** mis.h  -  Definitions and prototypes of the misula module.
**
** ---------------------------------------------------------------
*/

#ifndef _MIS_H
#define _MIS_H

/*@{*/
/**
** @name Module for the misula functions
**
** @date    23-Dez-2005
** @author  Paula Vilela
** @version 1.0
**
** This header file contains public definitions for the module
** of misula functions.   This module contains functions that
** that compute fix-end reaction moments and stiffness
** coefficients for a beam element with a cross secton 
** that has a fourth order polynomial variation of moment 
** of inertia.
**
** Revisions:
** - Report modifications here (including name and date).
**
*/

/*
** ---------------------------------------------------------------
** Used global definitions:
**
*/

/*
** ---------------------------------------------------------------
** Public data type definitions:
*/


/*
** ---------------------------------------------------------------
** Public functions:
**
*/

/* --------------------------------------------------------------- */
/**
** This function computes the rotational stiffness coefficients and 
** the transmition coefficient of moment from one extremity to the 
** other, for a beam element with a cross section that has fourth
** order polynomial variation of moment of inertia.
** Sign convention for the returned moment stiffness coefficients 
** is such that a positive value corresponds to the counter-clockwise
** direction.
**
**  @param len     - beam element length                                 (in)
**  @param J0      - initial moment of inertia value                     (in)
**  @param J1      - moment of inertia at 1/4 of beam span               (in)
**  @param J2      - moment of inertia at 1/2 of beam span               (in)
**  @param J3      - moment of inertia at 3/4 of beam span               (in)
**  @param J4      - final moment of inertia value                       (in)
**  @param Ka      - stiffness coefficient for an initial rotation       (out)
**  @param Kb      - stiffness coefficient for an final rotation         (out)
**  @param tab     - transm. coeff. of mom. from begin to the end of beam(out)
**  @param tba     - transm. coeff. of mom. from end to the begin of beam(out)
**
*/
void misRotStiffCoef( double len, double J0, double J1, 
                      double J2, double J3,  double J4,
                      double *Ka, double *Kb, double *tab, double *tba );

/* --------------------------------------------------------------- */
/**
** This function computes the fix-end moment reactions of a beam
** element with a cross section that has a fourth order polynomial
** variation of moment of inertia for an applied linearly distributed
** transversal load.
** Sign convention for transveral loads is such that a positive 
** value corresponds to an upward (in y direction) load.  Sign 
** convention for the returned moment reactions is such that a
** positive value corresponds to the counter-clockwise direction.
**
**  @param len     - beam element length                      (in)
**  @param qa      - initial transversal load value           (in)
**  @param qb      - final transversal load value             (in)
**  @param J0      - initial moment of inertia value          (in)
**  @param J1      - moment of inertia at 1/4 of beam span    (in)
**  @param J2      - moment of inertia at 1/2 of beam span    (in)
**  @param J3      - moment of inertia at 3/4 of beam span    (in)
**  @param J4      - final moment of inertia value            (in)
**  @param Ma      - initial fix-end moment reaction          (out)
**  @param Mb      - final fix-end moment reaction            (out)
**
*/
void misFixEnd_q( double len, double qa, double qb, 
                  double J0, double J1, double J2, double J3, double J4,
                  double *Ma, double *Mb );

/* --------------------------------------------------------------- */
/**
** This function computes the fix-end moment reactions of a beam
** element with a cross section that has a fourth order polynomial
** variation of moment of inertia for an imposed unitary discontinuity
** of the transversal displacement at any beam section.
** Sign convention for the unitary transversal displacement 
** discontinuity is that a positive value corresponds to an upward 
** (in y direction) discontinuity.
** Sign convention for the returned moment reactions is such that a
** positive value corresponds to the counter-clockwise direction.
**
**  @param len     - beam element length                      (in)
**  @param J0      - initial moment of inertia value          (in)
**  @param J1      - moment of inertia at 1/4 of beam span    (in)
**  @param J2      - moment of inertia at 1/2 of beam span    (in)
**  @param J3      - moment of inertia at 3/4 of beam span    (in)
**  @param J4      - final moment of inertia value            (in)
**  @param Ma      - initial fix-end moment reaction          (out)
**  @param Mb      - final fix-end moment reaction            (out)
**
*/
void misFixEnd_dv( double len, double J0, 
                   double J1, double J2, double J3, double J4,
                   double *Ma, double *Mb );

/* --------------------------------------------------------------- */
/**
** This function computes the fix-end moment reactions of a beam
** element with a cross section that has a fourth order polynomial
** variation of moment of inertia for an imposed unitary discontinuity
** of rotation at a given beam section.
** Sign convention for the unitary rotation discontinuity is that 
** a positive value corresponds to a counter-clockwise discontinuity.
** Sign convention for the returned moment reactions is such that a
** positive value corresponds to the counter-clockwise direction.
**
**  @param len     - beam element length                      (in)
**  @param xs      - discontinuity section position           (in)
**  @param J0      - initial moment of inertia value          (in)
**  @param J1      - moment of inertia at 1/4 of beam span    (in)
**  @param J2      - moment of inertia at 1/2 of beam span    (in)
**  @param J3      - moment of inertia at 3/4 of beam span    (in)
**  @param J4      - final moment of inertia value            (in)
**  @param Ma      - initial fix-end moment reaction          (out)
**  @param Mb      - final fix-end moment reaction            (out)
**
*/
void misFixEnd_dtheta( double len, double xs, 
                       double J0, double J1, double J2, double J3, double J4,
                       double *Ma, double *Mb );

/* --------------------------------------------------------------- */
/**
** This function computes the generic component of the transversal
** displacement at a given section location of a fix-ends beam 
** element with a cross section that has a fourth order polynomial
** variation of moment of inertia.  The generic component 
** corresponds to the transversal displacement due to the fix-end
** moment reactions that are given as arguments.
** Sign convention for the given moment reactions is such that a
** positive value corresponds to the counter-clockwise direction.
**
**  @param xs      - section location                         (in)
**  @param len     - beam element length                      (in)
**  @param J0      - initial moment of inertia value          (in)
**  @param J1      - moment of inertia at 1/4 of beam span    (in)
**  @param J2      - moment of inertia at 1/2 of beam span    (in)
**  @param J3      - moment of inertia at 3/4 of beam span    (in)
**  @param J4      - final moment of inertia value            (in)
**  @param Ma      - initial fix-end moment reaction          (in)
**  @param Mb      - final fix-end moment reaction            (in)
**
*/
double misDispl_MaMb( double xs, double len, 
                      double J0, double J1, double J2, double J3, double J4,
                      double Ma, double Mb );

/* --------------------------------------------------------------- */
/**
** This function computes the transversal displacement at a given 
** section location of a simply suported beam element with a cross
** section that has a fourth order polynomial variation of moment
** of inertia for an applyied linearly distributed transversal
** load.
** Sign convention for transveral loads is such that a positive 
** value corresponds to an upward (in y direction) load.
**
**  @param xs      - section location                         (in)
**  @param len     - beam element length                      (in)
**  @param J0      - initial moment of inertia value          (in)
**  @param J1      - moment of inertia at 1/4 of beam span    (in)
**  @param J2      - moment of inertia at 1/2 of beam span    (in)
**  @param J3      - moment of inertia at 3/4 of beam span    (in)
**  @param J4      - final moment of inertia value            (in)
**  @param qa      - initial transversal load value           (in)
**  @param qb      - final transversal load value             (in)
**
*/
double misDispl_q( double xs, double len, 
                   double J0, double J1, double J2, double J3, double J4,
                   double qa, double qb );

/* --------------------------------------------------------------- */
/**
** This function returns linearly interpolated intermediate
** section depth values for given end section depth values.
** The interpolated values correspond to values at 1/4, 1/2, and
** 3/4 of the beam span.
**
**  @param h0      - initial section depth value              (in)
**  @param h4      - final section depth value                (in)
**  @param h1      - section depth at 1/4 of beam span        (out)
**  @param h2      - section depth at 1/2 of beam span        (out)
**  @param h3      - section depth at 3/4 of beam span        (out)
**
*/
void misLinearInterpDepth( double h0, double h4,
                           double *h1, double *h2, double *h3 );

/* --------------------------------------------------------------- */
/**
** This function returns parabolic interpolated intermediate
** section depth values for given end section depth values.
** The interpolated values correspond to values at 1/4, 1/2, and
** 3/4 of the beam span.  The interporlation is such that the 
** parabola has a null derivative at the final end.
**
**  @param h0      - initial section depth value              (in)
**  @param h4      - final section depth value                (in)
**  @param h1      - section depth at 1/4 of beam span        (out)
**  @param h2      - section depth at 1/2 of beam span        (out)
**  @param h3      - section depth at 3/4 of beam span        (out)
**
*/
void misParabInterpDepth0( double h0, double h4,
                           double *h1, double *h2, double *h3 );

/* --------------------------------------------------------------- */
/**
** This function returns parabolic interpolated intermediate
** section depth values for given end section depth values.
** The interpolated values correspond to values at 1/4, 1/2, and
** 3/4 of the beam span.  The interporlation is such that the 
** parabola has a null derivative at the initial end.
**
**  @param h0      - initial section depth value              (in)
**  @param h4      - final section depth value                (in)
**  @param h1      - section depth at 1/4 of beam span        (out)
**  @param h2      - section depth at 1/2 of beam span        (out)
**  @param h3      - section depth at 3/4 of beam span        (out)
**
*/
void misParabInterpDepth1( double h0, double h4,
                           double *h1, double *h2, double *h3 );

/*@}*/

#endif
